/* kb.h -- interface of the 'libkb' keyboard library
  version 0.21, 01 Jan 1996.

  Copyright (C) 1995, 1996 Markus Franz Xaver Johannes Oberhumer

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Markus F.X.J. Oberhumer
  markus.oberhumer@jk.uni-linz.ac.at
 */


#ifndef __MFX_KB_H
#define __MFX_KB_H

#define __MFX_LIBKB

#define KB_VERSION_STRING   "0.21 BETA"
#define KB_VERSION_DATE     "01 Jan 1996"

#include <conio.h>

#ifdef __cplusplus
extern "C" {
#endif


/* utility macros for bits accessing */
#define KB_ALL_MASK(x,mask)		(((x) & (mask)) == (mask))
#define KB_ANY_MASK(x,mask)		(((x) & (mask)) != 0)
#define KB_NO_MASK(x,mask)		(((x) & (mask)) == 0)


/***********************************************************************
// INT9 keyboard access
************************************************************************/

/* installation of keyboard interrupt handler */
int kb_install(unsigned flags);		/* install interrupt handler */
void kb_remove(void);				/* remove interrupt handler */

/* read a single keypress */
unsigned kb_keypress(void);			/* basic access function */
int kb_kbhit(void);					/* compatible to kbhit() */
int kb_getch(void);					/* mostly compatible to getch() */
unsigned long kb_inkey(void);

/* access all keys */
#define kb_key(scan)				_kb_key[scan]
#define kb_shift()					_kb_shift
#define kb_shift_and(s)				_kb_shift &= (s)
#define kb_shift_or(s)				_kb_shift |= (s)
#define kb_keys_pressed()			_kb_keys_pressed
#define kb_flags()					_kb_flags


/* you are allowed to modify the following vars */
extern volatile unsigned short _kb_shift;	/* current shift state */

/* these are READ ONLY - use the access-macros above when possible */
extern int _kb_mode;						/* do we have a keyboard handler? */
extern unsigned _kb_flags;					/* flags from kb_install() */
extern volatile unsigned char _kb_key[128];	/* the key flags */
extern volatile int _kb_keys_pressed;		/* number of keys pressed */


/* get the name of a key */
/* const char *kb_keyname(int scan); NOT YET IMPLEMENTED */


/***********************************************************************
// flags - kb_install() and kb_flags()
************************************************************************/

/* pass these to kb_install() */
#define KB_FLAG_NO_ATEXIT           0x0001   /* do not use atexit() */
#define KB_FLAG_LOCALIZE            0x0002   /* use local key mappings */
#define KB_FLAG_INIT_SHIFT_TOGGLES  0x0004   /* init current shift toggles */

/* these information bits are set by kb_install() */
#define KB_FLAG_ATEXIT_INSTALLED    0x0100   /* atexit() installed */
#define KB_FLAG_LOCK_FAILED         0x0200   /* error while locking memory */


/***********************************************************************
// keyboard scan codes - kb_key()
************************************************************************/

/* real keys */
#define KB_SCAN_ESC             1
#define KB_SCAN_1               2 
#define KB_SCAN_2               3 
#define KB_SCAN_3               4
#define KB_SCAN_4               5
#define KB_SCAN_5               6
#define KB_SCAN_6               7
#define KB_SCAN_7               8
#define KB_SCAN_8               9
#define KB_SCAN_9               10
#define KB_SCAN_0               11
#define KB_SCAN_MINUS           12
#define KB_SCAN_EQUAL           13
#define KB_SCAN_BACKSPACE       14
#define KB_SCAN_TAB             15 
#define KB_SCAN_Q               16
#define KB_SCAN_W               17
#define KB_SCAN_E               18
#define KB_SCAN_R               19
#define KB_SCAN_T               20
#define KB_SCAN_Y               21
#define KB_SCAN_U               22
#define KB_SCAN_I               23
#define KB_SCAN_O               24
#define KB_SCAN_P               25
#define KB_SCAN_OPENBRACE       26
#define KB_SCAN_CLOSEBRACE      27
#define KB_SCAN_ENTER           28
#define KB_SCAN_RETURN          KB_SCAN_ENTER
#define KB_SCAN_LCONTROL        29    /* Left Control */
#define KB_SCAN_A               30
#define KB_SCAN_S               31
#define KB_SCAN_D               32
#define KB_SCAN_F               33
#define KB_SCAN_G               34
#define KB_SCAN_H               35
#define KB_SCAN_J               36
#define KB_SCAN_K               37
#define KB_SCAN_L               38
#define KB_SCAN_COLON           39
#define KB_SCAN_QUOTE           40
#define KB_SCAN_TILDE           41
#define KB_SCAN_LSHIFT          42   /* Left Shift */
#define KB_SCAN_BACKSLASH       43
#define KB_SCAN_Z               44
#define KB_SCAN_X               45
#define KB_SCAN_C               46
#define KB_SCAN_V               47
#define KB_SCAN_B               48
#define KB_SCAN_N               49
#define KB_SCAN_M               50
#define KB_SCAN_COMMA           51
#define KB_SCAN_PERIOD          52
#define KB_SCAN_SLASH           53
#define KB_SCAN_RSHIFT          54   /* Right Shift */
#define KB_SCAN_ASTERISK_PAD    55
#define KB_SCAN_ALT             56   /* left Alt */
#define KB_SCAN_LALT            KB_SCAN_ALT
#define KB_SCAN_SPACE           57
#define KB_SCAN_CAPSLOCK        58
#define KB_SCAN_F1              59
#define KB_SCAN_F2              60
#define KB_SCAN_F3              61
#define KB_SCAN_F4              62
#define KB_SCAN_F5              63
#define KB_SCAN_F6              64
#define KB_SCAN_F7              65
#define KB_SCAN_F8              66
#define KB_SCAN_F9              67
#define KB_SCAN_F10             68
#define KB_SCAN_NUMLOCK         69
#define KB_SCAN_SCRLOCK         70
#define KB_SCAN_7_PAD           71
#define KB_SCAN_8_PAD           72
#define KB_SCAN_9_PAD           73
#define KB_SCAN_MINUS_PAD       74
#define KB_SCAN_4_PAD           75
#define KB_SCAN_5_PAD           76
#define KB_SCAN_6_PAD           77
#define KB_SCAN_PLUS_PAD        78
#define KB_SCAN_1_PAD           79
#define KB_SCAN_2_PAD           80
#define KB_SCAN_3_PAD           81
#define KB_SCAN_0_PAD           82
#define KB_SCAN_COMMA_PAD       83
#define KB_SCAN_PRINT           84 
				     /* FIXME: where is 85 ? */
#define KB_SCAN_LESS            86   /* this is on my German keyboard */
#define KB_SCAN_F11             87
#define KB_SCAN_F12             88


/* real (but prefixed) keys mapped to unique scancodes */
#define KB_SCAN_ENTER_PAD       96    /* on keypad */
#define KB_SCAN_RCONTROL        97    /* Right Control */
#define KB_SCAN_SLASH_PAD       98    /* on keypad */
#define KB_SCAN_ALTGR           100   /* Alt Graphics = right Alt */
#define KB_SCAN_RALT            KB_SCAN_ALTGR
#define KB_SCAN_HOME            102   /* KB_SCAN_7_PAD */
#define KB_SCAN_UP              103   /* KB_SCAN_8_PAD */
#define KB_SCAN_PGUP            104   /* KB_SCAN_9_PAD */
#define KB_SCAN_LEFT            105   /* KB_SCAN_4_PAD */
#define KB_SCAN_RIGHT           106   /* KB_SCAN_6_PAD */
#define KB_SCAN_END             107   /* KB_SCAN_1_PAD */
#define KB_SCAN_DOWN            108   /* KB_SCAN_2_PAD */
#define KB_SCAN_PGDN            109   /* KB_SCAN_3_PAD */
#define KB_SCAN_INSERT          110   /* KB_SCAN_0_PAD */
#define KB_SCAN_DELETE          111   /* KB_SCAN_COMMA_PAD */
#define KB_SCAN_DEL             KB_SCAN_DELETE

/* #define KB_SCAN_CONTROL_BACKSLASH 99 - Linux ?? */
/* #define KB_SCAN_BREAK           101 - Linux ?? */


/***********************************************************************
// keyboard shift states - kb_shift()
************************************************************************/

/* shift flags */
#define KB_SHIFT_LSHIFT         0x0001
#define KB_SHIFT_RSHIFT         0x0002
#define KB_SHIFT_LCONTROL       0x0004
#define KB_SHIFT_RCONTROL       0x0008
#define KB_SHIFT_ALT            0x0010  
#define KB_SHIFT_ALTGR          0x0020 
#define KB_SHIFT_LALT           KB_SHIFT_ALT
#define KB_SHIFT_RALT           KB_SHIFT_ALTGR

/* these are toggles - they only change when the key is pressed */
#define KB_SHIFT_CAPSLOCK       0x0040
#define KB_SHIFT_NUMLOCK        0x0080
#define KB_SHIFT_SCRLOCK        0x0100
#define KB_SHIFT_INSERT         0x0200
#define KB_SHIFT_PAUSE          0x0400   /* this key has no scan code */
#define KB_SHIFT_CONTROL_BREAK  0x0800   /* this key has no scan code */

/* flags for masking multiple shift keys */
#define KB_SHIFT_ANY            0x0fff
#define KB_SHIFT_ANY_SHIFT      (KB_SHIFT_LSHIFT | KB_SHIFT_RSHIFT)
#define KB_SHIFT_ANY_CONTROL    (KB_SHIFT_LCONTROL | KB_SHIFT_RCONTROL)
#define KB_SHIFT_ANY_ALT        (KB_SHIFT_ALT | KB_SHIFT_ALTGR)


/***********************************************************************
// BIOS keyboard access
************************************************************************/

int kb_bios_kbhit(void);
int kb_bios_getch(void);			/* compatible to getch() */


/***********************************************************************
// a compatible kbhit()/getch() implementation for emx+gcc
************************************************************************/

#if defined(__EMX__)
#undef kbhit
#undef getch
int kbhit(void);
int getch(void);
#endif



#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* already included */

/*
vi:ts=4
*/
